@@ -418,7 +418,7 @@ class NaPoleonXConnector(object):
             productCodes=productCodes,
             minDate=minDate,
             maxDate=maxDate,
-            lastOnly = True
+            lastOnly = False
         )
 
     def get_product_data(self, productCodes):
@@ -469,6 +469,12 @@ class NaPoleonXConnector(object):
             df.loc[sub_df.index, asset_col] = sub_df.loc[:, asset_col]
         return df
 
+
+    def get_vm_rates_dataframe_from_currency(self, currencies, minDate, maxDate):
+        vm_rates_product_codes = [f'VM-EUR-{currency}' for currency in currencies if currency != 'EUR']
+        return self.get_dataframe(vm_rates_product_codes, minDate, maxDate)
+
+
     def get_vm_rates_dataframe(self, productCodes, minDate, maxDate):
         currencies = set([self.get_currency(product_code) for product_code in productCodes])
         vm_rates_product_codes = [f'VM-EUR-{currency}' for currency in currencies if currency != 'EUR']
